/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 */
using System;
using NAnt.Core;
using NAnt.NUnit2.Tasks;
using NAnt.Core.Tasks;
using NAnt.Core.Attributes;
using TopCoder.Test.Framework.Servers;

namespace TopCoder.Test.Framework
{
    /// <summary>
    /// <p>
    /// Defines a class which represents a NAnt task that extends the optional NUnit2 task to provide support
    /// for testing using this Component. This is the main NAnt task of this component.
    /// </p>
    /// <p>
    /// The attributes supported by this task are:
    /// <ol>
    ///     <li>none additional to the supported by &quot;nunit2&quot; task.</li>
    ///     <li>&quot;server&quot; : should be at least one time, represents a particular server.</li>
    /// </ol>
    /// </p>
    /// </summary>
    /// <remarks>
    /// This class is not thread-safe since it is mutable.
    /// </remarks>
    ///
    /// <author>real_vg</author>
    /// <author>TCSDEVELOPER</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (c) 2006, TopCoder, Inc. All rights reserved.</copyright>
    [TaskName("serverapptest")]
    public class ServerAppTestTask : NUnit2Task
    {
        /// <summary>
        /// Represents the array of nested 'server' XML elements. Each server element describes a particular server
        /// to be started/initialized while running tests. The 'serverapptest' NAnt task should include
        /// at least one nested 'server' XML element so this array will contain at least one element.
        /// </summary>
        private ServerElement[] servers = null;

        /// <summary>
        /// Gets the array of nested 'server' XML elements. Each server element describes a particular server
        /// to be started/initialized while running tests. The 'serverapptest' NAnt task should include
        /// at least one nested 'server' XML element so this array will contain at least one element.
        /// </summary>
        /// <value>The array of nested 'server' XML elements. Each server element describes a particular server
        /// to be started/initialized while running tests.</value>
        [BuildElementArray("server", ElementType=typeof(ServerElement), Required=true)]
        public ServerElement[] Servers
        {
            get
            {
                return this.servers;
            }
            set
            {
                this.servers = value;
            }
        }

        /// <summary>
        /// Creates a new instance of <c>ServerAppTestTask</c> class.
        /// </summary>
        public ServerAppTestTask()
            : base()
        {
        }

        /// <summary>
        /// Executes the task.
        /// </summary>
        /// <remarks>
        /// First starts the servers if needed. Then runs the NUnit testcases. After running the testcases the
        /// servers are stopped.
        /// </remarks>
        /// <exception cref="ServerTaskException">If any error happens(this exception wll contain also the array
        /// of inner exceptions).</exception>
        protected override void ExecuteTask()
        {
            Exception[] innerExceptions = new Exception[servers.Length];

            // Startup the servers
            StartsServers(innerExceptions);


            // Initialize the servers
            InitializeServers(innerExceptions);


            // Run the tests, just call the method of superclass
            try
            {
                base.ExecuteTask();
            }
            catch (BuildException e)
            {
                StopsServers(true, true, innerExceptions);
                throw new ServerTaskException("Error occurs when process the base's excuteTask method.",
                    new Exception[] { e });
            }

            // Deinitialize the servers
            // If any of the servers fails to deinitialize/stop (ServerException is caught),
            StopsServers(true, false, innerExceptions);
        }

        /// <summary>
        /// Starts the servers. If any of the servers fails to start (ServerException is caught), the
        /// servers that were started should be stopped
        /// </summary>
        /// <param name="innerExceptions">The exceptions array to store all the exceptions when starting the servers.
        /// </param>
        /// <exception cref="ServerTaskException">If any error happens(this exception wll contain also the array
        /// of inner exceptions).</exception>
        private void StartsServers(Exception[] innerExceptions)
        {
            bool hasException = false;
            for (int i = 0; i < this.servers.Length; i++)
            {
                try
                {
                    Server server = servers[i].GetServerInstance();
                    if (servers[i].StartServer)
                    {
                        server.Project = Project;
                        server.StartUp();
                    }
                }
                catch (ServerException e)
                {
                    innerExceptions[i] = e;
                    hasException = true;
                }
            }
            if (hasException)
            {
                // if any of the servers fails to start (ServerException is caught), the servers that were started
                // should be stopped
                StopsServers(false, true, innerExceptions);
                throw new ServerTaskException("Errors occur while processing the startUp of servers.",
                    innerExceptions);
            }
        }

        /// <summary>
        /// Initializes the servers. If any of the servers fails to initialize (ServerException is caught), the
        /// servers that were started should be stopped, the servers that were initialized should be dinitialized
        /// </summary>
        /// <param name="innerExceptions">The exceptions array to store all the exceptions when initializing the
        /// servers.</param>
        /// <exception cref="ServerTaskException">If any error happens(this exception wll contain also the array
        /// of inner exceptions).</exception>
        private void InitializeServers(Exception[] innerExceptions)
        {
            bool hasException = false;
            for (int i = 0; i < this.servers.Length; i++)
            {
                try
                {
                    Server server = servers[i].GetServerInstance();
                    if (servers[i].InitServer)
                    {
                        server.InitData();
                    }
                }
                catch (ServerException e)
                {
                    innerExceptions[i] = e;
                    hasException = true;
                }
            }
            if (hasException)
            {
                // if any of the servers fails to initialize (ServerException is caught), the servers that were
                // started should be stopped, the servers that were initialized should be dinitialized
                StopsServers(true, true, innerExceptions);
                throw new ServerTaskException("Errors occur while processing the initData of servers.",
                    innerExceptions);
            }
        }

        /// <summary>
        /// Deinitializes/shutdowns the servers. If any of the servers fails to deinitialize/stop
        /// (ServerException is caught), the remaining servers should be stopped.
        /// </summary>
        /// <param name="cleanUpData">Whether to deinitialize the servers.</param>
        /// <param name="egnoreException">Whether to ingore the exception thrown in the Deinitialize/shutdown
        /// process.</param>
        /// <param name="innerExceptions">The exceptions array to store all the exceptions when deinitializing or
        /// shutdowning the servers.</param>
        private void StopsServers(bool cleanUpData, bool egnoreException, Exception[] innerExceptions)
        {
            bool hasException = false;
            for (int i = this.servers.Length - 1; i >= 0; --i)
            {
                if (cleanUpData)
                {
                    try
                    {
                        Server server = servers[i].GetServerInstance();
                        if (this.servers[i].InitServer)
                        {
                            server.CleanUpData();
                        }
                    }
                    catch (ServerException e)
                    {
                        if (!egnoreException)
                        {
                            innerExceptions[i] = e;
                            hasException = true;
                        }
                    }
                }

                // if we merge the CleanUpData and shutdown processes, and exceptions are thrown from the
                // CleanUpData, The ShutDown is not processed at all. So we seperate the two processes
                try
                {
                    Server server = servers[i].GetServerInstance();
                    if (this.servers[i].StartServer)
                    {
                        server.ShutDown();
                        server.Project = null;
                    }
                }
                catch (ServerException e)
                {
                    if (!egnoreException)
                    {
                        // the field will not be null when exceptions thrown in CleanUpData, and we should not
                        // replace it.
                        if (innerExceptions[i] == null)
                        {
                            innerExceptions[i] = e;
                        }
                        hasException = true;
                    }
                }
            }

            if (hasException)
            {
                throw new ServerTaskException("Errors occur when deinitialize/shutdown of servers.",
                    innerExceptions);
            }
        }
    }
}
