/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 */
using System;
using System.Reflection;
using NAnt.Core;
using NAnt.Core.Types;
using NAnt.Core.Attributes;
using TopCoder.Test.Framework.Servers;

namespace TopCoder.Test.Framework
{
    /// <summary>
    /// <p>
    /// Defines a structure which holds the 'server' XML element to be used in the NAnt build file.
    /// This element should be used inside the 'serverapptes' NAnt task which is represented by the
    /// <c>ServerAppTestTask</c> class.
    /// </p>
    /// <p>
    /// The attributes supported by this element are:
    /// <ol>
    ///     <li>&quot;class&quot; - this attribute represents the class name of the server
    ///     described by this element. This attribute is required.</li>
    ///     <li>&quot;startserver&quot; - this attribute specifies if the server should be started
    ///     by the Component when running tests. This attribute is not required, and its
    ///     default value is <c>true</c>.</li>
    ///     <li>&quot;initserver&quot; - this attribute specifies if the server should be
    ///     intialized with data by the Component when running tests. This attribute is
    ///     not required, and its default value is <c>true</c>.</li>
    /// </ol>
    /// </p>
    /// <p>
    /// The nested XML elements supported are:
    /// <ol>
    ///     <li>&quot;option&quot; - can be used zero or one times, represents a particular option of the
    ///     server.</li>
    ///     <li>&quot;credential&quot; - can be used zero or one times, represents a credentials used to login
    ///     to the server.</li>
    ///     <li>&quot;sourcefiles&quot; - can be used zero or more times, represents the files used to initialize
    ///     the server with data.</li>
    /// </ol>
    /// </p>
    /// </summary>
    /// <remarks>
    /// This class is not thread-safe since it is mutable.
    /// </remarks>
    ///
    /// <author>real_vg</author>
    /// <author>TCSDEVELOPER</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (c) 2006, TopCoder, Inc. All rights reserved.</copyright>
    [ElementName("server")]
    public class ServerElement : Element
    {
        /// <summary>
        /// Represents the 'class' attribute. This attribute represents the class name of the server described
        /// by this element. This attribute is required.
        /// </summary>
        private string className = null;

        /// <summary>
        /// Represents the server described by this element.
        /// </summary>
        private Server server = null;

        /// <summary>
        /// Represents the value of 'startserver' attribute.
        /// </summary>
        /// <remarks>
        /// The 'startserver' attribute specifies if the server should be started by the Component when
        /// running tests. This attribute is not required, and its default value is <c>true</c>.
        /// </remarks>
        private bool startServer = true;

        /// <summary>
        /// Represents the value of 'initserver' attribute.
        /// </summary>
        /// <remarks>
        /// The 'initserver' attribute specifies if the server should be intialized with data by the Component
        /// when running tests. This attribute is not required, and its default value is <c>true</c>.
        /// </remarks>
        private bool initServer = true;

        /// <summary>
        /// Represents the collection of nested 'option' XML elements, which represent the options of the server.
        /// The options can be any key-value pairs, where keys and values are strings. This collection is
        /// required.
        /// </summary>
        private OptionCollection options = new OptionCollection();

        /// <summary>
        /// Represents the nested 'credentials' XML element, which represents the credentials used to
        /// authenticate to the server.
        /// </summary>
        /// <remarks>
        /// The username and password of a domain is included in this element. This nested element can possibly
        /// be ommited, which depends on the particular server.
        /// </remarks>
        private Credential credentials = new Credential();

        /// <summary>
        /// Represents the nested 'sourcefiles' XML element, which represents the files used when initializing
        /// the server with data.
        /// </summary>
        /// <remarks>
        /// This can be an application to deploy for an application server or some SQL scripts, DB tables, etc.
        /// for a data server, or some other files the particular server may need to be initialized.
        /// This nested element can possibly be ommited if the derived from <c>Server</c> class permits it to
        /// be ommited.
        /// </remarks>
        private FileSet sourceFiles = new FileSet();

        /// <summary>
        /// Gets or sets the 'class' attribute. This attribute represents the class name of the server
        /// described by this element. This attribute is required.
        /// </summary>
        /// <value>The 'class' attribute.</value>
        [TaskAttribute("class", Required = true)]
        public string ClassName
        {
            get
            {
                return this.className;
            }
            set
            {
                this.className = value;
            }
        }

        /// <summary>
        /// Gets or sets the 'startserver' attribute. The attribute specifies if the server should
        /// be started by the Component when running tests. This attribute is not required and its default value
        /// is <c>true</c>.
        /// </summary>
        /// <value>The 'startserver' attribute.</value>
        [TaskAttribute("startserver")]
        public bool StartServer
        {
            get
            {
                return this.startServer;
            }
            set
            {
                this.startServer = value;
            }
        }

        /// <summary>
        /// Gets or sets the 'initserver' attribute. The attribute specifies if the server should
        /// be intialized with data by the Component when running tests. This attribute is not required and
        /// its default value is <c>true</c>.
        /// </summary>
        /// <value>The 'initserver' attribute.</value>
        [TaskAttribute("initserver")]
        public bool InitServer
        {
            get
            {
                return this.initServer;
            }
            set
            {
                this.initServer = value;
            }
        }

        /// <summary>
        /// Gets the collection of nested 'option' XML elements, which represent the options of
        /// the server. The options can be any key-value pairs, where keys and values are strings. This
        /// collection is required.
        /// </summary>
        /// <value>The collection of nested 'option' XML elements, which represent the options of the server.
        /// </value>
        [BuildElementCollection("options", "option")]
        public OptionCollection Options
        {
            get
            {
                return this.options;
            }
        }

        /// <summary>
        /// Gets or sets the nested 'credentials' XML element, which represents the credentials used to
        /// authenticate to the server.
        /// </summary>
        /// <value>The nested 'credentials' XML element, which represents the credentials used to authenticate
        /// to the server.</value>
        [BuildElement("credentials")]
        public Credential Credentials
        {
            get
            {
                return this.credentials;
            }
            set
            {
                this.credentials = value;
            }
        }

        /// <summary>
        /// Represents the nested 'sourcefiles' XML element, which represents the files used when initializing
        /// the server with data.
        /// </summary>
        /// <value>The nested 'sourcefiles' XML element, which represents the files used when initializing
        /// the server with data.</value>
        [BuildElement("sourcefiles")]
        public FileSet SourceFiles
        {
            get
            {
                return this.sourceFiles;
            }
            set
            {
                this.sourceFiles = value;
            }
        }

        /// <summary>
        /// Creates a new instance of <c>ServerElement</c> class.
        /// </summary>
        public ServerElement()
        {
        }

        /// <summary>
        /// Gets the <c>Server</c> instance described by this XML element.
        /// </summary>
        /// <returns>A <c>Server</c> instance described by this XML element.</returns>
        /// <exception cref="ServerException">If any error happens when creating an instance of <c>Server</c>.
        /// </exception>
        public Server GetServerInstance()
        {
            if (this.server != null)
            {
                return this.server;
            }

            // Create an instance of the class
            this.server = (Server) this.CreateInstance(ClassName, typeof(Server));

            // Copy the property values
            this.server.StartServer = this.StartServer;
            this.server.InitServer = this.InitServer;
            this.server.Options = this.Options;
            this.server.Credentials = this.Credentials;
            this.server.SourceFiles = this.SourceFiles;

            // Return the created instance
            return this.server;
        }

        /// <summary>
        /// Creates a new instance of the class. The type name of the class is given. The parameter of the
        /// constructor is also given. The created instance must be an instance of the expected type.
        /// </summary>
        /// <param name="typeName">The type name of the class to be instantiated.</param>
        /// <param name="expected">The expected type of the created instance.</param>
        /// <returns>A new instance of the given class.</returns>
        /// <exception cref="ServerException">A new instance of the given type cannot be
        /// created; or the created instance is not an instance of the expected type.</exception>
        private object CreateInstance(string typeName, Type expected)
        {
            if (typeName == null)
            {
                throw new ServerException("The class name should not be null.");
            }

            Type type;

            try
            {
                // Get the type
                type = Type.GetType(typeName, true);
            }
            catch (TargetInvocationException e)
            {
                throw new ServerException("The static initializer throws exception.", e);
            }
            catch (TypeLoadException e)
            {
                throw new ServerException("The type cannot be loaded.", e);
            }
            catch (ArgumentException e)
            {
                throw new ServerException("The type name is illegal.", e);
            }

            // Check if the expected is assignable from the retrieved type
            if (!expected.IsAssignableFrom(type))
            {
                throw new ServerException("The type should be sub-class of " +
                    expected.ToString() + ".");
            }

            try
            {
                return Activator.CreateInstance(type);
            }
            catch (ArgumentException e)
            {
                throw new ServerException("Type is not a RuntimeType.", e);
            }
            catch (MemberAccessException e)
            {
                throw new ServerException("The class is an abstract class.", e);
            }
            catch (TargetInvocationException e)
            {
                throw new ServerException("The constructor throws exception.", e);
            }
            catch (NotSupportedException e)
            {
                throw new ServerException("Special class cannot be instantiated.", e);
            }
            catch (Exception e)
            {
                throw new ServerException("Error to initialize the instance.", e);
            }
        }
    }
}