/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 */
using System;
using NAnt.Core.Types;

namespace TopCoder.Test.Framework.Servers
{
    /// <summary>
    /// <p>
    /// Defines an abstract base class derived from <c>Server</c> class for all the classes which represent a
    /// data server to be used while running tests. This class is a base for another component which will provide
    /// testing ability for database servers.
    /// </p>
    /// <p>
    /// The supported options are:
    /// DB_DRIVER - specifies the name of the database driver to use for connecting to database.
    /// DB_CONNECTION_STR - specifies the database connection string to use for connecting to database.
    /// SQL_INIT_SCRIPT - specifies the name of file, containing a SQL script used to init the data.
    /// SQL_CLEANUP_SCRIPT - specifies the name of file, containing a SQL script used to clean-up the data.
    /// </p>
    /// </summary>
    /// <remarks>
    /// This class is not thread-safe since its parent class is mutable.
    /// </remarks>
    ///
    /// <author>real_vg</author>
    /// <author>TCSDEVELOPER</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (c) 2006, TopCoder, Inc. All rights reserved.</copyright>
    public abstract class DatabaseServer : Server
    {
        /// <summary>
        /// Represents the name(key) of the db_driver option. The value of the db_driver option
        /// represents the name of the database driver for connecting to database.
        /// </summary>
        public static readonly string DB_DRIVER = "db_driver";

        /// <summary>
        /// Represents the name(key) of the db_connection_str option. The value of the db_connection_str option
        /// represents the database connection string for connecting to database.
        /// </summary>
        public static readonly string DB_CONNECTION_STR = "db_connection_str";

        /// <summary>
        /// Represents the name(key) of the sql_init_script option. The value of the sql_init_script option
        /// represents the name of file which contains a SQL script used to init the data.
        /// </summary>
        public static readonly string SQL_INIT_SCRIPT = "sql_init_script";

        /// <summary>
        /// Represents the name(key) of the sql_cleanup_script option. The value of the sql_cleanup_script option
        /// represents the name of file which contains a SQL script used to clean-up the data.
        /// </summary>
        public static readonly string SQL_CLEANUP_SCRIPT = "sql_cleanup_script";

        /// <summary>
        /// Gets the value of the db_driver option.
        /// </summary>
        /// <value>The value of the db_driver option.</value>
        /// <exception cref="ServerException">If the option value is missing or an empty string.</exception>
        public string DatabaseDriver
        {
            get
            {
                return this.GetOption(DB_DRIVER);
            }
        }

        /// <summary>
        /// Gets the value of the db_connection_str option.
        /// </summary>
        /// <value>The value of the db_connection_str option.</value>
        /// <exception cref="ServerException">If the option value is missing or an empty string.</exception>
        public string DatabaseConnectionString
        {
            get
            {
                return this.GetOption(DB_CONNECTION_STR);
            }
        }

        /// <summary>
        /// Gets the value of the sql_init_script option.
        /// </summary>
        /// <value>The value of the sql_init_script option.</value>
        /// <exception cref="ServerException">If the option value is missing or an empty string.</exception>
        public string SqlInitScript
        {
            get
            {
                return this.GetOption(SQL_INIT_SCRIPT);
            }
        }

        /// <summary>
        /// Gets the value of the sql_cleanup_script option.
        /// </summary>
        /// <value>The value of the sql_cleanup_script option.</value>
        /// <exception cref="ServerException">If the option value is missing or an empty string.</exception>
        public string SqlCleanUpScript
        {
            get
            {
                return this.GetOption(SQL_CLEANUP_SCRIPT);
            }
        }

        /// <summary>
        /// Gets the option value from the options dictionary with the given option name.
        /// </summary>
        /// <param name="optionName">The option name to retrieve the option value.</param>
        /// <returns>The retrieved option value.</returns>
        /// <exception cref="ServerException">If the option value is missing or an empty string.</exception>
        private string GetOption(string optionName)
        {
            Option option = Options[optionName];
            if (option == null)
            {
                throw new ServerException("The option value for " + optionName + " is missing.");
            }
            if (option.Value.Trim().Length == 0)
            {
                throw new ServerException("The option value for " + optionName + " is empty.");
            }
            return option.Value;
        }

        /// <summary>
        /// Creates a new instance of <c>DatabaseServer</c> class.
        /// </summary>
        protected DatabaseServer() : base()
        {
        }
    }
}