/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 */
using System;
using NAnt.Core;
using NAnt.Core.Types;

namespace TopCoder.Test.Framework.Servers
{
    /// <summary>
    /// <p>
    /// Defines an abstract base class for all the classes which represent a server to be used when running tests.
    /// </p>
    /// <p>
    /// This class will hold some attributes passed from the <c>ServerElement</c> class which can be accessed
    /// via the get/set properties. This class provides the abstract methods of starting/stopping/initializing
    /// data/clean-up, which should be implemented by the derived classes.
    /// </p>
    /// <p>
    /// The supported options are:
    /// TIMEOUT - specifies the time waiting for the server to run.
    /// PORT - specifies the port to connect to the server.
    /// ADDRESS - specifies the address of the machine which runs the server.
    /// </p>
    /// </summary>
    /// <remarks>
    /// This class is not thread-safe since it is mutable.
    /// </remarks>
    ///
    /// <author>real_vg</author>
    /// <author>TCSDEVELOPER</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (c) 2006, TopCoder, Inc. All rights reserved.</copyright>
    public abstract class Server
    {
        /// <summary>
        /// Represents the value of 'startserver' attribute.
        /// </summary>
        /// <remarks>
        /// The 'startserver' attribute specifies if the server should be started by the Component when
        /// running tests. This attribute is not required, and its default value is <c>true</c>.
        /// </remarks>
        private bool startServer = true;

        /// <summary>
        /// Represents the value of 'initserver' attribute.
        /// </summary>
        /// <remarks>
        /// The 'initserver' attribute specifies if the server should be intialized with data by the Component
        /// when running tests. This attribute is not required, and its default value is <c>true</c>.
        /// </remarks>
        private bool initServer = true;

        /// <summary>
        /// Represents the collection of nested 'option' XML elements, which represent the options of the server.
        /// The options can be any key-value pairs, where keys and values are strings.
        /// </summary>
        /// <remarks>
        /// The string constants representing the names of the most common options are provided by this class.
        /// They are: <c>TIMEOUT</c>, <c>PORT</c> and <c>ADDRESS</c>.
        /// </remarks>
        private OptionCollection options = null;

        /// <summary>
        /// Represents the nested 'credentials' XML element, which represents the credentials used to
        /// authenticate to the server.
        /// </summary>
        /// <remarks>
        /// The username and password of a domain is included in this element. This nested element can possibly
        /// be ommited, which depends on the particular server.
        /// </remarks>
        private Credential credentials = null;

        /// <summary>
        /// Represents the nested 'sourcefiles' XML element, which represents the files used when initializing
        /// the server with data.
        /// </summary>
        /// <remarks>
        /// This can be an application to deploy for an application server or some SQL scripts, DB tables, etc.
        /// for a data server, or some other files the particular server may need to be initialized.
        /// This nested element can possibly be ommited if the derived from <c>Server</c> class permits it to
        /// be ommited.
        /// </remarks>
        private FileSet sourceFiles = null;

        /// <summary>
        /// Represents the project which belongs to this server.
        /// </summary>
        private Project project = null;

        /// <summary>
        /// Gets or sets the value of 'startserver' attribute.
        /// </summary>
        /// <value>The value of 'startserver' attribute.</value>
        public bool StartServer
        {
            get
            {
                return this.startServer;
            }
            set
            {
                this.startServer = value;
            }
        }

        /// <summary>
        /// Gets or sets the value of 'initserver' attribute.
        /// </summary>
        /// <value>The value of 'initserver' attribute.</value>
        public bool InitServer
        {
            get
            {
                return this.initServer;
            }
            set
            {
                this.initServer = value;
            }
        }

        /// <summary>
        /// Gets or sets the collection of nested 'option' XML elements, which represent the options
        /// of the server. The options can be any key-value pairs, where keys and values are strings.
        /// </summary>
        /// <value>The collection of nested 'option' XML elements, which represent the options of the server.
        /// </value>
        public OptionCollection Options
        {
            get
            {
                return this.options;
            }
            set
            {
                this.options = value;
            }
        }

        /// <summary>
        /// Gets or sets the nested 'credentials' XML element, which represents the credentials used to
        /// authenticate to the server.
        /// </summary>
        /// <value>The nested 'credentials' XML element, which represents the credentials used to authenticate
        /// to the server.</value>
        public Credential Credentials
        {
            get
            {
                return this.credentials;
            }
            set
            {
                this.credentials = value;
            }
        }

        /// <summary>
        /// Gets or sets the nested 'sourcefiles' XML element, which represents the files used when initializing
        /// the server with data.
        /// </summary>
        /// <value>The nested 'sourcefiles' XML element, which represents the files used when initializing the
        /// server with data.</value>
        public FileSet SourceFiles
        {
            get
            {
                return this.sourceFiles;
            }
            set
            {
                this.sourceFiles = value;
            }
        }

        /// <summary>
        /// Gets or sets the project which belongs to this server.
        /// </summary>
        /// <value>The project which belongs to this server.</value>
        public Project Project
        {
            get
            {
                return this.project;
            }
            set
            {
                this.project = value;
            }
        }

        /// <summary>
        /// Represents the name(key) of the timeout option. The value of the timeout option(which is a time in
        /// milliseconds) will be used in the <c>StartUp()</c> method of the specific <c>Server</c> class.
        /// If the specified time passes and the server is not running the <c>StartUp()</c> method, exception
        /// will be thrown.
        /// </summary>
        public static readonly string TIMEOUT = "timeout";

        /// <summary>
        /// Represents the name(key) of the port option. The value of the port option will be used to connect
        /// the server.
        /// </summary>
        public static readonly string PORT = "port";

        /// <summary>
        /// Represents the name(key) of the address option. The value of the address option represents the address
        /// of the machine which runs the server.
        /// </summary>
        public static readonly string ADDRESS = "address";

        /// <summary>
        /// Gets the value of the timeout option.
        /// </summary>
        /// <value>The value of the timeout option.</value>
        /// <exception cref="ServerException">If the option value is missing or the valud of the timeout is not
        /// non-negative.</exception>
        public int Timeout
        {
            get
            {
                return GetNonNegetiveIntValue(GetOption(TIMEOUT));
            }
        }

        /// <summary>
        /// Gets the value of the port option.
        /// </summary>
        /// <value>The value of the port option.</value>
        /// <exception cref="ServerException">If the option value is missing or the value is not a valid
        /// TCP/IP port.</exception>
        public int Port
        {
            get
            {
                int port = GetNonNegetiveIntValue(GetOption(PORT));
                if (port > 65535)
                {
                    throw new ServerException("The value of port is not a valid TCP/IP port.");
                }
                return port;
            }
        }

        /// <summary>
        /// Gets the value of the address option.
        /// </summary>
        /// <value>The value of the address option.</value>
        /// <exception cref="ServerException">If the option value is missing or an empty string.</exception>
        public string Address
        {
            get
            {
                return GetOption(ADDRESS);
            }
        }

        /// <summary>
        /// Gets the option value from the options dictionary with the given option name.
        /// </summary>
        /// <param name="optionName">The option name to retrieve the option value.</param>
        /// <returns>The retrieved option value.</returns>
        /// <exception cref="ServerException">If the option value is missing or an empty string.</exception>
        private string GetOption(string optionName)
        {
            Option option = this.options[optionName];
            if (option == null)
            {
                throw new ServerException("The option value for " + optionName + " is missing.");
            }
            if (option.Value.Trim().Length == 0)
            {
                throw new ServerException("The option value for " + optionName + " is empty.");
            }
            return option.Value;
        }

        /// <summary>
        /// Parses a non-negetive integer from the given string value.
        /// </summary>
        /// <param name="stringValue">The given string value where the non-negetive integer is retrieved.</param>
        /// <returns>A non-negetive integer from the given string value.</returns>
        /// <exception cref="ServerException">The string value is not a non-negative integer.</exception>
        private int GetNonNegetiveIntValue(string stringValue)
        {
            int value;

            try
            {
                value = Int32.Parse(stringValue);
            }
            catch (FormatException e)
            {
                throw new ServerException(stringValue  + " is invalid.", e);
            }
            catch (OverflowException e)
            {
                throw new ServerException(stringValue + " is invalid.", e);
            }

            // Check non-negetive
            if (value < 0)
            {
                throw new ServerException(stringValue + " must be non-negative.");
            }

            return value;
        }

        /// <summary>
        /// Creates a new instance of <c>Server</c> class.
        /// </summary>
        protected Server()
        {
        }

        /// <summary>
        /// Starts the server. Before starting the server, the web-application should be deployed first to a
        /// specific directory according the pre-defined option value.
        /// </summary>
        /// <remarks>
        /// In the case when 'startserver' attribute is set to false, the server should not be started.
        /// </remarks>
        /// <exception cref="ServerException">If it fails to start the server.</exception>
        public abstract void StartUp();

        /// <summary>
        /// Stops the server.
        /// </summary>
        /// <remarks>
        /// The server should not be stopped if it wasn't started by the <c>startUp()</c> method.
        /// </remarks>
        /// <exception cref="ServerException">If it fails to shut down the server.</exception>
        public abstract void ShutDown();

        /// <summary>
        /// Initializes the server with data.
        /// </summary>
        /// <remarks>
        /// In the case when 'initserver' attribute is set to false, does nothing, or can check if the data
        /// are properly initialized,
        /// </remarks>
        /// <exception cref="ServerException">If it fails to initialize the server with data.</exception>
        public abstract void InitData();

        /// <summary>
        /// Clean-ups the data initialized by the <c>initData()</c> method.
        /// </summary>
        /// <remarks>
        /// In the case when 'initserver' attribute is set to false or data was not initialized, does nothing.
        /// </remarks>
        /// <exception cref="ServerException">If it fails to clean-up the data initialized by the
        /// <c>initData()</c> method.</exception>
        public abstract void CleanUpData();
    }
}