/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 */
using System;
using System.IO;
using System.ServiceProcess;
using NAnt.Core;
using NAnt.Core.Types;
using NUnit.Framework;

namespace TopCoder.Test.Framework
{
    /// <summary>
    /// Tests the functionality and error cases of <c>ServerAppTestTask</c> class.
    /// </summary>
    /// <author>TCSDEVELOPER</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (c) 2006, TopCoder, Inc. All rights reserved.</copyright>
    [TestFixture]
    public class ServerAppTestTaskUnitTest
    {
        /// <summary>
        /// Represents default value for the SERVICE_NAME option. It the name of IIS service.
        /// </summary>
        private static readonly string DEFAULT_SERVICE_NAME = "w3svc";

        /// <summary>
        /// Represents default value for the TIMEOUT option. It is represented in milliseconds.
        /// </summary>
        private static readonly int DEFAULT_TIMEOUT = 60000;

        /// <summary>
        /// Represents the destination directory to deploy.
        /// </summary>
        private static readonly string DES = @"../../test_files/deploy";

        /// <summary>
        /// Represents the <c>ServerAppTestTask</c> instance used in tests.
        /// </summary>
        private ServerAppTestTask task = null;

        /// <summary>
        /// Sets up the test environment. The instances for testing will be created.
        /// </summary>
        [SetUp]
        public void SetUp()
        {
            task = new MockServerAppTestTask();
        }

        /// <summary>
        /// Tests the accuracy of constructor <c>ServerAppTestTask</c>.
        /// </summary>
        [Test]
        public void TestServerAppTestTask_Accuracy()
        {
            Assert.IsNotNull(task, "The task should be created properly.");
        }

        /// <summary>
        /// Tests the <c>ServerAppTestTask</c> when the serve elements were missing, expect BuildException.
        /// </summary>
        [Test, ExpectedException(typeof(BuildException))]
        public void TestInitialize_MissingServers()
        {
            Project project = new Project(@"../../test_files/Invalid/MissingServer.xml", Level.Info);
            task.Project = project;
            task.Initialize(UnitTestHelper.GetXmlNode(project.BuildFileLocalName, "serverapptest"));
        }

        /// <summary>
        /// Tests the accuracy of <c>Initialize</c>. All the properties will be loaded from
        /// xml node.
        /// </summary>
        [Test]
        public void TestInitialize_LoadFromXmlNodeAccuracy()
        {
            Project project = new Project(@"../../test_files/ServerapptestElement.xml", Level.Info);
            task.Project = project;
            task.Initialize(UnitTestHelper.GetXmlNode(project.BuildFileLocalName, "serverapptest"));

            // check
            Assert.AreEqual(2, task.Servers.Length, "The servers retrieved should be correct.");

            // check the first element
            ServerElement element = task.Servers[0];
            Assert.AreEqual("TopCoder.Test.Framework.Servers.Web.IISServer",
                element.ClassName, "The servers retrieved should be correct.");

            Assert.AreEqual(true, element.InitServer, "The servers retrieved should be correct.");

            Assert.AreEqual(true, element.StartServer, "The servers retrieved should be correct.");

            OptionCollection collection = element.Options;
            Assert.AreEqual(3, collection.Count, "The servers retrieved should be correct.");
            Assert.AreEqual("w3svc", collection["service_name"].Value, "The servers retrieved should be correct.");
            Assert.AreEqual("test_files/Web", collection["deployment_dir"].Value,
                "The servers retrieved should be correct.");
            Assert.AreEqual("60000", collection["timeout"].Value,
                "The servers retrieved should be correct.");

            FileSet fileSet = element.SourceFiles;
            Assert.AreEqual(new DirectoryInfo("../../test_files/src").FullName, fileSet.BaseDirectory,
                "The servers retrieved should be correct.");
            Assert.AreEqual(true, fileSet.DefaultExcludes, "The servers retrieved should be correct.");

            // check the second element
            element = task.Servers[1];
            Assert.AreEqual("TopCoder.Test.Framework.Servers.MockDatabaseServer", element.ClassName,
                "The servers retrieved should be correct.");

            Assert.AreEqual(true, element.InitServer, "The servers retrieved should be correct.");

            Assert.AreEqual(false, element.StartServer, "The servers retrieved should be correct.");

            collection = element.Options;
            Assert.AreEqual(4, collection.Count, "The servers retrieved should be correct.");
            Assert.AreEqual("some_port_number", collection["port"].Value,
                "The servers retrieved should be correct.");
            Assert.AreEqual("some_server_address", collection["address"].Value,
                "The servers retrieved should be correct.");
            Assert.AreEqual("test_files/sql/init.sql", collection["sql_init_script"].Value,
                "The servers retrieved should be correct."); ;
            Assert.AreEqual("test_files/sql/cleanup.sql", collection["sql_cleanup_script"].Value,
                "The servers retrieved should be correct.");

            Credential credential = element.Credentials;
            Assert.AreEqual("topcoder", credential.UserName, "The servers retrieved should be correct.");
            Assert.AreEqual("some_password", credential.Password, "The servers retrieved should be correct.");
        }

        /// <summary>
        /// Tests the accuracy of <c>Servers</c> Property.
        /// </summary>
        [Test]
        public void TestServers_Accuracy()
        {
            Assert.AreEqual(null, task.Servers, "The servers field should be got properly.");
            ServerElement[] elements = new ServerElement[] { new ServerElement() };
            task.Servers = elements;
            Assert.AreEqual(elements, task.Servers, "The servers field should be set properly.");
        }

        /// <summary>
        /// Tests the <c>ExecuteTask()</c> when the service_name is invalid, expect ServerTaskException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerTaskException))]
        public void TestExecuteTask_InvalidServiceName()
        {
            // stop the service first
            ServiceController serviceController = new ServiceController(DEFAULT_SERVICE_NAME);
            if (serviceController.Status != ServiceControllerStatus.Stopped)
            {
                serviceController.Stop();
                serviceController.WaitForStatus(ServiceControllerStatus.Stopped,
                    TimeSpan.FromMilliseconds(DEFAULT_TIMEOUT));
            }

            Project project = new Project(@"../../test_files/Invalid/InvalidServiceName.xml", Level.Info);
            task.Project = project;
            task.Initialize(UnitTestHelper.GetXmlNode(project.BuildFileLocalName, "serverapptest"));

            try
            {
                (task as MockServerAppTestTask).ExecuteTask();
            }
            finally
            {
                // check that the server has stoped
                serviceController = new ServiceController(DEFAULT_SERVICE_NAME);
                Assert.AreEqual(ServiceControllerStatus.Stopped, serviceController.Status,
                    "The service should be stopped.");

                // check the web-application has been deleted
                Assert.AreEqual(0, new DirectoryInfo(DES).GetFiles().Length,
                    "The web-application should be deleted.");
            }
        }

        /// <summary>
        /// Tests the <c>ExecuteTask()</c> when the SourceFiles is missing, expect ServerTaskException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerTaskException))]
        public void TestExecuteTask_MissingSourceFiles()
        {
            // stop the service first
            ServiceController serviceController = new ServiceController(DEFAULT_SERVICE_NAME);
            if (serviceController.Status != ServiceControllerStatus.Stopped)
            {
                serviceController.Stop();
                serviceController.WaitForStatus(ServiceControllerStatus.Stopped,
                    TimeSpan.FromMilliseconds(DEFAULT_TIMEOUT));
            }

            Project project = new Project(@"../../test_files/Invalid/MissingSourceFiles.xml", Level.Info);
            task.Project = project;
            task.Initialize(UnitTestHelper.GetXmlNode(project.BuildFileLocalName, "serverapptest"));

            try
            {
                (task as MockServerAppTestTask).ExecuteTask();
            }
            finally
            {
                // check that the server has stoped
                serviceController = new ServiceController(DEFAULT_SERVICE_NAME);
                Assert.AreEqual(ServiceControllerStatus.Stopped, serviceController.Status,
                    "The service should be stopped.");

                // check the web-application has been deleted
                Assert.AreEqual(0, new DirectoryInfo(DES).GetFiles().Length,
                    "The web-application should be deleted.");
            }
        }

        /// <summary>
        /// Tests the <c>ExecuteTask()</c> when the DeploymentDir is missing, expect ServerTaskException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerTaskException))]
        public void TestExecuteTask_MissingDeploymentDir()
        {
            // stop the service first
            ServiceController serviceController = new ServiceController(DEFAULT_SERVICE_NAME);
            if (serviceController.Status != ServiceControllerStatus.Stopped)
            {
                serviceController.Stop();
                serviceController.WaitForStatus(ServiceControllerStatus.Stopped,
                    TimeSpan.FromMilliseconds(DEFAULT_TIMEOUT));
            }

            Project project = new Project(@"../../test_files/Invalid/MissingDeploymentDir.xml", Level.Info);
            task.Project = project;
            task.Initialize(UnitTestHelper.GetXmlNode(project.BuildFileLocalName, "serverapptest"));

            try
            {
                (task as MockServerAppTestTask).ExecuteTask();
            }
            finally
            {
                // check that the server has stoped
                serviceController = new ServiceController(DEFAULT_SERVICE_NAME);
                Assert.AreEqual(ServiceControllerStatus.Stopped, serviceController.Status,
                    "The service should be stopped.");

                // check the web-application has been deleted
                Assert.AreEqual(0, new DirectoryInfo(DES).GetFiles().Length,
                    "The web-application should be deleted.");
            }
        }

        /// <summary>
        /// Tests the <c>ExecuteTask()</c> when the TIMEOUT is invalid, expect ServerTaskException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerTaskException))]
        public void TestExecuteTask_InvalidTimeOut1()
        {
            // stop the service first
            ServiceController serviceController = new ServiceController(DEFAULT_SERVICE_NAME);
            if (serviceController.Status != ServiceControllerStatus.Stopped)
            {
                serviceController.Stop();
                serviceController.WaitForStatus(ServiceControllerStatus.Stopped,
                    TimeSpan.FromMilliseconds(DEFAULT_TIMEOUT));
            }

            Project project = new Project(@"../../test_files/Invalid/InvalidTimeOut1.xml", Level.Info);
            task.Project = project;
            task.Initialize(UnitTestHelper.GetXmlNode(project.BuildFileLocalName, "serverapptest"));

            try
            {
                (task as MockServerAppTestTask).ExecuteTask();
            }
            finally
            {
                // check that the server has stoped
                serviceController = new ServiceController(DEFAULT_SERVICE_NAME);
                Assert.AreEqual(ServiceControllerStatus.Stopped, serviceController.Status,
                    "The service should be stopped.");

                // check the web-application has been deleted
                Assert.AreEqual(0, new DirectoryInfo(DES).GetFiles().Length,
                    "The web-application should be deleted.");
            }
        }

        /// <summary>
        /// Tests the <c>ExecuteTask()</c> when the TIMEOUT is invalid, expect ServerTaskException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerTaskException))]
        public void TestExecuteTask_InvalidTimeOut2()
        {
            // stop the service first
            ServiceController serviceController = new ServiceController(DEFAULT_SERVICE_NAME);
            if (serviceController.Status != ServiceControllerStatus.Stopped)
            {
                serviceController.Stop();
                serviceController.WaitForStatus(ServiceControllerStatus.Stopped,
                    TimeSpan.FromMilliseconds(DEFAULT_TIMEOUT));
            }

            Project project = new Project(@"../../test_files/Invalid/InvalidTimeOut2.xml", Level.Info);
            task.Project = project;
            task.Initialize(UnitTestHelper.GetXmlNode(project.BuildFileLocalName, "serverapptest"));

            try
            {
                (task as MockServerAppTestTask).ExecuteTask();
            }
            finally
            {
                // check that the server has stoped
                serviceController = new ServiceController(DEFAULT_SERVICE_NAME);
                Assert.AreEqual(ServiceControllerStatus.Stopped, serviceController.Status,
                    "The service should be stopped.");

                // check the web-application has been deleted
                Assert.AreEqual(0, new DirectoryInfo(DES).GetFiles().Length,
                    "The web-application should be deleted.");
            }
        }

        /// <summary>
        /// Tests the <c>ExecuteTask()</c> when the TIMEOUT is invalid, expect ServerTaskException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerTaskException))]
        public void TestExecuteTask_InvalidTimeOut3()
        {
            // stop the service first
            ServiceController serviceController = new ServiceController(DEFAULT_SERVICE_NAME);
            if (serviceController.Status != ServiceControllerStatus.Stopped)
            {
                serviceController.Stop();
                serviceController.WaitForStatus(ServiceControllerStatus.Stopped,
                    TimeSpan.FromMilliseconds(DEFAULT_TIMEOUT));
            }

            Project project = new Project(@"../../test_files/Invalid/InvalidTimeOut3.xml", Level.Info);
            task.Project = project;
            task.Initialize(UnitTestHelper.GetXmlNode(project.BuildFileLocalName, "serverapptest"));

            try
            {
                (task as MockServerAppTestTask).ExecuteTask();
            }
            finally
            {
                // check that the server has stoped
                serviceController = new ServiceController(DEFAULT_SERVICE_NAME);
                Assert.AreEqual(ServiceControllerStatus.Stopped, serviceController.Status,
                    "The service should be stopped.");

                // check the web-application has been deleted
                Assert.AreEqual(0, new DirectoryInfo(DES).GetFiles().Length,
                    "The web-application should be deleted.");
            }
        }

        /// <summary>
        /// Tests the accuracy of <c>ExecuteTask()</c>.
        /// </summary>
        [Test]
        public void TestExecuteTask_Accuracy()
        {
            // stop the service first
            ServiceController serviceController = new ServiceController(DEFAULT_SERVICE_NAME);
            if (serviceController.Status != ServiceControllerStatus.Stopped)
            {
                serviceController.Stop();
                serviceController.WaitForStatus(ServiceControllerStatus.Stopped,
                    TimeSpan.FromMilliseconds(DEFAULT_TIMEOUT));
            }

            Project project = new Project(@"../../test_files/ServerapptestElement.xml", Level.Info);
            task.Project = project;
            task.Initialize(UnitTestHelper.GetXmlNode(project.BuildFileLocalName, "serverapptest"));

            (task as MockServerAppTestTask).ExecuteTask();

            // check that the server has stoped
            serviceController = new ServiceController(DEFAULT_SERVICE_NAME);
            Assert.AreEqual(ServiceControllerStatus.Stopped, serviceController.Status,
                "The service should be stopped.");

            // check the web-application has been deleted
            Assert.AreEqual(0, new DirectoryInfo(DES).GetFiles().Length,
                "The web-application should be deleted.");
        }
    }
}
