/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 */
using System;
using System.IO;
using NAnt.Core;
using NAnt.Core.Types;
using NUnit.Framework;
using TopCoder.Test.Framework.Servers;
using TopCoder.Test.Framework.Servers.Web;

namespace TopCoder.Test.Framework
{
    /// <summary>
    /// Tests the functionality and error cases of <c>ServerElement</c> class.
    /// </summary>
    /// <author>TCSDEVELOPER</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (c) 2006, TopCoder, Inc. All rights reserved.</copyright>
    [TestFixture]
    public class ServerElementUnitTest
    {
        /// <summary>
        /// Represents the <c>ServerElement</c> instance used in tests.
        /// </summary>
        private ServerElement element = null;

        /// <summary>
        /// Sets up the test environment. The instances for testing will be created.
        /// </summary>
        [SetUp]
        public void SetUp()
        {
            element = new ServerElement();
        }

        /// <summary>
        /// Tests the accuracy of constructor <c>ServerElement</c>.
        /// </summary>
        [Test]
        public void TestServerElement_Accuracy()
        {
            Assert.IsNotNull(element, "The element should be created properly.");
        }

        /// <summary>
        /// Tests the accuracy of <c>Initialize</c>. All the properties will be loaded from
        /// xml node.
        /// </summary>
        [Test]
        public void TestInitialize_LoadFromXmlNodeAccuracy()
        {
            Project project = new Project(@"../../test_files/ServerElement.xml", Level.Info);
            element.Project = project;
            element.Initialize(UnitTestHelper.GetXmlNode(project.BuildFileLocalName, "server"));

            // check
            Assert.AreEqual("SomeDataBaseServer", element.ClassName, "The className field should be set.");

            Assert.AreEqual(true, element.InitServer, "The initServer field should be set.");

            Assert.AreEqual(false, element.StartServer, "The startServer field should be set.");

            OptionCollection collection = element.Options;
            Assert.AreEqual(2, collection.Count, "The options field should be set.");
            Assert.AreEqual("some_port_number", collection["port"].Value, "The options field should be set.");
            Assert.AreEqual("some_server_address", collection["address"].Value,
                "The options field should be set.");

            Credential credential = element.Credentials;
            Assert.AreEqual("topcoder", credential.UserName, "The credentials field should be set.");
            Assert.AreEqual("some_password", credential.Password, "The credentials field should be set.");

            FileSet fileSet = element.SourceFiles;
            Assert.AreEqual(new DirectoryInfo("../../test_files/src").FullName, fileSet.BaseDirectory,
                "The sourceFiles field should be set.");
            Assert.AreEqual(true, fileSet.DefaultExcludes, "The sourceFiles field should be set.");
        }

        /// <summary>
        /// Tests the accuracy of <c>ClassName</c> Property.
        /// </summary>
        [Test]
        public void TestClassName_Accuracy()
        {
            Assert.AreEqual(null, element.ClassName, "The className field should be got properly.");
            element.ClassName = "foo";
            Assert.AreEqual("foo", element.ClassName, "The className field should be set properly.");
        }

        /// <summary>
        /// Tests the accuracy of <c>StartServer</c> Property.
        /// </summary>
        [Test]
        public void TestStartServer_Accuracy()
        {
            Assert.AreEqual(true, element.StartServer, "The startServer field should be got properly.");
            element.StartServer = false;
            Assert.AreEqual(false, element.StartServer, "The startServer field should be set properly.");
        }

        /// <summary>
        /// Tests the accuracy of <c>InitServer</c> Property.
        /// </summary>
        [Test]
        public void TestInitServer_Accuracy()
        {
            Assert.AreEqual(true, element.InitServer, "The initServer field should be got properly.");
            element.InitServer = false;
            Assert.AreEqual(false, element.InitServer, "The initServer field should be set properly.");
        }

        /// <summary>
        /// Tests the accuracy of <c>Options</c> Property.
        /// </summary>
        [Test]
        public void TestOptions_Accuracy()
        {
            Assert.IsNotNull(element.Options, "The options field should be got properly.");
        }

        /// <summary>
        /// Tests the accuracy of <c>Credentials</c> Property.
        /// </summary>
        [Test]
        public void TestCredentials_Accuracy()
        {
            Assert.IsNotNull(element.Credentials, "The credentials field should be got properly.");
            Credential credential = new Credential();
            element.Credentials = credential;
            Assert.AreEqual(credential, element.Credentials, "The credentials field should be set properly.");
        }

        /// <summary>
        /// Tests the accuracy of <c>SourceFiles</c> Property.
        /// </summary>
        [Test]
        public void TestSourceFiles_Accuracy()
        {
            Assert.IsNotNull(element.SourceFiles, "The sourceFiles field should be got properly.");
            FileSet sourceFiles = new FileSet();
            element.SourceFiles = sourceFiles;
            Assert.AreEqual(sourceFiles, element.SourceFiles, "The sourceFiles field should be set properly.");
        }

        /// <summary>
        /// Tests the <c>GetServerInstance()</c> when className is null, expect ServerException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestGetServerInstance_NullClassName()
        {
            element.GetServerInstance();
        }

        /// <summary>
        /// Tests the <c>GetServerInstance()</c> when className is empty, expect ServerException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestGetServerInstance_EmptyClassName()
        {
            element.ClassName = " ";
            element.GetServerInstance();
        }

        /// <summary>
        /// Tests the <c>GetServerInstance()</c> when className is invalid, expect ServerException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestGetServerInstance_InvalidClassName1()
        {
            element.ClassName = typeof(ApplicationServer).FullName;
            element.GetServerInstance();
        }

        /// <summary>
        /// Tests the <c>GetServerInstance()</c> when className is invalid, expect ServerException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestGetServerInstance_InvalidClassName2()
        {
            element.ClassName = typeof(Server).FullName;
            element.GetServerInstance();
        }

        /// <summary>
        /// Tests the <c>GetServerInstance()</c> when className is invalid, expect ServerException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestGetServerInstance_InvalidClassName3()
        {
            element.ClassName = typeof(string).FullName;
            element.GetServerInstance();
        }

        /// <summary>
        /// Tests the accuracy of <c>GetServerInstance()</c>.
        /// </summary>
        [Test]
        public void TestGetServerInstance_Accuracy()
        {
            element.ClassName = typeof(IISServer).FullName;
            Server server = element.GetServerInstance();

            // check
            Assert.IsNotNull(server, "The server instance should be got properly.");
            Assert.AreEqual(element.InitServer, server.InitServer, "The initServer field should be set.");
            Assert.AreEqual(element.StartServer, server.StartServer, "The startServer field should be set.");
            Assert.AreEqual(element.Options, server.Options, "The options field should be set.");
            Assert.AreEqual(element.Credentials, server.Credentials, "The credentials field should be set.");
            Assert.AreEqual(element.SourceFiles, server.SourceFiles, "The sourceFiles field should be set.");

            // retrieve again
            Assert.AreSame(server, element.GetServerInstance(), "The server instance should be got properly.");
        }
    }
}
