/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 */
using System;
using System.IO;
using System.Collections;
using System.ServiceProcess;
using NAnt.Core;
using NAnt.Core.Types;
using NUnit.Framework;

namespace TopCoder.Test.Framework.Servers.Web
{
    /// <summary>
    /// Tests the functionality and error cases of <c>IISServer</c> class.
    /// </summary>
    /// <author>TCSDEVELOPER</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (c) 2006, TopCoder, Inc. All rights reserved.</copyright>
    [TestFixture]
    public class IISServerUnitTest : ApplicationServerUnitTest
    {
        /// <summary>
        /// Represents default value for the SERVICE_NAME option. It the name of IIS service.
        /// </summary>
        private static readonly string DEFAULT_SERVICE_NAME = "w3svc";

        /// <summary>
        /// Represents default value for the TIMEOUT option. It is represented in milliseconds.
        /// </summary>
        private static readonly int DEFAULT_TIMEOUT = 60000;

        /// <summary>
        /// Represents the src directory to deploy.
        /// </summary>
        private static readonly string SRC = @"../../test_files/src";

        /// <summary>
        /// Represents the destination directory to deploy.
        /// </summary>
        private static readonly string DES = @"../../test_files/deploy";

        /// <summary>
        /// Initiates <see cref="ServerUnitTest.meType"/>.
        /// </summary>
        public IISServerUnitTest()
        {
            meType = typeof(IISServer);
        }

        /// <summary>
        /// Tests the accuracy of the constructor.
        /// </summary>
        [Test]
        public void TestIISServer_Accuracy()
        {
            Assert.IsNotNull(server, "Constructor should be correct.");
        }

        /// <summary>
        /// Tests the <c>StartUp</c> when the Project is missing,
        /// expect ServerException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestStartUp_MissingProject()
        {
            InitializeServer();

            // start up the server
            ServiceController serviceController = new ServiceController(DEFAULT_SERVICE_NAME);
            if (serviceController.Status != ServiceControllerStatus.Stopped)
            {
                serviceController.Stop();
                serviceController.WaitForStatus(ServiceControllerStatus.Stopped,
                    TimeSpan.FromMilliseconds(DEFAULT_TIMEOUT));
            }

            // set the file set
            server.Project = null;

            server.StartUp();
        }

        /// <summary>
        /// Tests the <c>StartUp</c> when the SourceFiles is missing,
        /// expect ServerException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestStartUp_MissingSourceFiles()
        {
            InitializeServer();

            // start up the server
            ServiceController serviceController = new ServiceController(DEFAULT_SERVICE_NAME);
            if (serviceController.Status != ServiceControllerStatus.Stopped)
            {
                serviceController.Stop();
                serviceController.WaitForStatus(ServiceControllerStatus.Stopped,
                    TimeSpan.FromMilliseconds(DEFAULT_TIMEOUT));
            }

            // set the file set
            server.SourceFiles = null;

            server.StartUp();
        }

        /// <summary>
        /// Tests the <c>StartUp</c> when the deployment dir property is missing,
        /// expect ServerException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestStartUp_MissingDeploymentDirPropertyValue()
        {
            InitializeServer();

            // start up the server
            ServiceController serviceController = new ServiceController(DEFAULT_SERVICE_NAME);
            if (serviceController.Status != ServiceControllerStatus.Stopped)
            {
                serviceController.Stop();
                serviceController.WaitForStatus(ServiceControllerStatus.Stopped,
                    TimeSpan.FromMilliseconds(DEFAULT_TIMEOUT));
            }

            OptionCollection collection = UnitTestHelper.CreateOptionCollection(
                new string[0], new string[0]);
            server.Options = collection;

            server.StartUp();
        }

        /// <summary>
        /// Tests the <c>StartUp</c> when the TIMEOUT property is invalid, expect ServerException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestStartUp_InvalidTimeOutPropertyValue1()
        {
            try
            {
                InitializeServer();

                // start up the server
                ServiceController serviceController = new ServiceController(DEFAULT_SERVICE_NAME);
                if (serviceController.Status != ServiceControllerStatus.Stopped)
                {
                    serviceController.Stop();
                    serviceController.WaitForStatus(ServiceControllerStatus.Stopped,
                        TimeSpan.FromMilliseconds(DEFAULT_TIMEOUT));
                }

                OptionCollection collection = UnitTestHelper.CreateOptionCollection(
                    new string[] { ApplicationServer.DEPLOYMENT_DIR, Server.TIMEOUT },
                    new string[] { @"test_files/deploy", "ab" });
                server.Options = collection;

                server.StartUp();
            }
            finally
            {
                try
                {
                    server.ShutDown();
                }
                catch
                {
                    // ignore
                }
            }
        }

        /// <summary>
        /// Tests the <c>StartUp</c> when the TIMEOUT property is invalid, expect ServerException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestStartUp_InvalidTimeOutPropertyValue2()
        {
            try
            {
                InitializeServer();

                // start up the server
                ServiceController serviceController = new ServiceController(DEFAULT_SERVICE_NAME);
                if (serviceController.Status != ServiceControllerStatus.Stopped)
                {
                    serviceController.Stop();
                    serviceController.WaitForStatus(ServiceControllerStatus.Stopped,
                        TimeSpan.FromMilliseconds(DEFAULT_TIMEOUT));
                }

                OptionCollection collection = UnitTestHelper.CreateOptionCollection(
                    new string[] { ApplicationServer.DEPLOYMENT_DIR, Server.TIMEOUT },
                    new string[] { @"test_files/deploy", "-1" });
                server.Options = collection;

                server.StartUp();
            }
            finally
            {
                try
                {
                    server.ShutDown();
                }
                catch
                {
                    // ignore
                }
            }
        }

        /// <summary>
        /// Tests the <c>StartUp</c> when the TIMEOUT property is invalid, expect ServerException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestStartUp_InvalidTimeOutPropertyValue3()
        {
            try
            {
                InitializeServer();

                // start up the server
                ServiceController serviceController = new ServiceController(DEFAULT_SERVICE_NAME);
                if (serviceController.Status != ServiceControllerStatus.Stopped)
                {
                    serviceController.Stop();
                    serviceController.WaitForStatus(ServiceControllerStatus.Stopped,
                        TimeSpan.FromMilliseconds(DEFAULT_TIMEOUT));
                }

                OptionCollection collection = UnitTestHelper.CreateOptionCollection(
                    new string[] { ApplicationServer.DEPLOYMENT_DIR, Server.TIMEOUT },
                    new string[] { @"test_files/deploy", "1111111111111111111111111" });
                server.Options = collection;

                server.StartUp();
            }
            finally
            {
                try
                {
                    server.ShutDown();
                }
                catch
                {
                    // ignore
                }
            }
        }

        /// <summary>
        /// Tests the <c>StartUp</c> when the service name property is invalid, expect ServerException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestStartUp_InvalidServiceNamePropertyValue()
        {
            try
            {
                InitializeServer();

                // start up the server
                ServiceController serviceController = new ServiceController(DEFAULT_SERVICE_NAME);
                if (serviceController.Status != ServiceControllerStatus.Stopped)
                {
                    serviceController.Stop();
                    serviceController.WaitForStatus(ServiceControllerStatus.Stopped,
                        TimeSpan.FromMilliseconds(DEFAULT_TIMEOUT));
                }

                OptionCollection collection = UnitTestHelper.CreateOptionCollection(
                    new string[] { ApplicationServer.DEPLOYMENT_DIR, IISServer.SERVICE_NAME },
                    new string[] { @"test_files/deploy", "Invalid" });
                server.Options = collection;

                server.StartUp();
            }
            finally
            {
                try
                {
                    server.ShutDown();
                }
                catch
                {
                    // ignore
                }
            }
        }

        /// <summary>
        /// Tests the <c>StartUp</c> when the service has already been started, expect ServerException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestStartUp_AlreadyStarted()
        {
            try
            {
                InitializeServer();

                // start up the server
                ServiceController serviceController = new ServiceController(DEFAULT_SERVICE_NAME);
                if (serviceController.Status != ServiceControllerStatus.Running)
                {
                    serviceController.Start();
                    serviceController.WaitForStatus(ServiceControllerStatus.Running,
                        TimeSpan.FromMilliseconds(DEFAULT_TIMEOUT));
                }
                server.StartUp();
            }
            finally
            {
                try
                {
                    server.ShutDown();
                }
                catch
                {
                    // ignore
                }
            }
        }

        /// <summary>
        /// Tests the accuracy of <c>StartUp</c>.
        /// </summary>
        [Test]
        public void TestStartUp_StoppedAccuracy()
        {
            InitializeServer();

            // start up the server
            ServiceController serviceController = new ServiceController(DEFAULT_SERVICE_NAME);
            if (serviceController.Status != ServiceControllerStatus.Stopped)
            {
                serviceController.Stop();
                serviceController.WaitForStatus(ServiceControllerStatus.Stopped,
                    TimeSpan.FromMilliseconds(DEFAULT_TIMEOUT));
            }
            server.StartUp();

            // check that the server has started
            serviceController = new ServiceController(DEFAULT_SERVICE_NAME);
            Assert.AreEqual(ServiceControllerStatus.Running, serviceController.Status,
                "The service should be started.");

            // check the web-application has been deployed
            CheckDeployResult();

            // shutdown the server
            server.ShutDown();
        }

        /// <summary>
        /// Tests the accuracy of <c>StartUp</c>.
        /// </summary>
        [Test]
        public void TestStartUp_PausedAccuracy()
        {
            InitializeServer();

            // start up the server
            ServiceController serviceController = new ServiceController(DEFAULT_SERVICE_NAME);
            if (serviceController.Status != ServiceControllerStatus.Running)
            {
                serviceController.Start();
                serviceController.WaitForStatus(ServiceControllerStatus.Running,
                    TimeSpan.FromMilliseconds(DEFAULT_TIMEOUT));
            }
            if (serviceController.Status != ServiceControllerStatus.Paused)
            {
                serviceController.Pause();
                serviceController.WaitForStatus(ServiceControllerStatus.Paused,
                    TimeSpan.FromMilliseconds(DEFAULT_TIMEOUT));
            }
            server.StartUp();

            // check that the server has started
            serviceController = new ServiceController(DEFAULT_SERVICE_NAME);
            Assert.AreEqual(ServiceControllerStatus.Running, serviceController.Status,
                "The service should be started.");


            // check the web-application has been deployed
            CheckDeployResult();

            // shutdown the server
            server.ShutDown();
        }

        /// <summary>
        /// Tests the <c>ShutDown</c> when the Project is missing,
        /// expect ServerException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestShutDown_MissingProject()
        {
            InitializeServer();

            // start up the server
            ServiceController serviceController = new ServiceController(DEFAULT_SERVICE_NAME);
            if (serviceController.Status != ServiceControllerStatus.Running)
            {
                serviceController.Start();
                serviceController.WaitForStatus(ServiceControllerStatus.Running,
                    TimeSpan.FromMilliseconds(DEFAULT_TIMEOUT));
            }

            server.Project = null;
            server.ShutDown();
        }

        /// <summary>
        /// Tests the <c>ShutDown</c> when the SourceFiles is missing,
        /// expect ServerException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestShutDown_MissingSourceFiles()
        {
            InitializeServer();

            // start up the server
            ServiceController serviceController = new ServiceController(DEFAULT_SERVICE_NAME);
            if (serviceController.Status != ServiceControllerStatus.Running)
            {
                serviceController.Start();
                serviceController.WaitForStatus(ServiceControllerStatus.Running,
                    TimeSpan.FromMilliseconds(DEFAULT_TIMEOUT));
            }

            server.SourceFiles = null;
            server.ShutDown();
        }

        /// <summary>
        /// Tests the <c>ShutDown</c> when the deployment dir property is missing,
        /// expect ServerException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestShutDown_MissingDeploymentDirPropertyValue()
        {
            InitializeServer();

            // start up the server
            ServiceController serviceController = new ServiceController(DEFAULT_SERVICE_NAME);
            if (serviceController.Status != ServiceControllerStatus.Running)
            {
                serviceController.Start();
                serviceController.WaitForStatus(ServiceControllerStatus.Running,
                    TimeSpan.FromMilliseconds(DEFAULT_TIMEOUT));
            }

            OptionCollection collection = UnitTestHelper.CreateOptionCollection(
                new string[0], new string[0]);
            server.Options = collection;
            server.ShutDown();
        }

        /// <summary>
        /// Tests the <c>ShutDown</c> when the TIMEOUT property is invalid, expect ServerException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestShutDown_InvalidTimeOutPropertyValue1()
        {
            InitializeServer();

            // start up the server
            ServiceController serviceController = new ServiceController(DEFAULT_SERVICE_NAME);
            if (serviceController.Status != ServiceControllerStatus.Running)
            {
                serviceController.Start();
                serviceController.WaitForStatus(ServiceControllerStatus.Running,
                    TimeSpan.FromMilliseconds(DEFAULT_TIMEOUT));
            }

            OptionCollection collection = UnitTestHelper.CreateOptionCollection(
                new string[] { ApplicationServer.DEPLOYMENT_DIR, Server.TIMEOUT },
                new string[] { @"test_files/deploy", "ab" });
            server.Options = collection;
            server.ShutDown();
        }

        /// <summary>
        /// Tests the <c>ShutDown</c> when the TIMEOUT property is invalid, expect ServerException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestShutDown_InvalidTimeOutPropertyValue2()
        {
            InitializeServer();

            // start up the server
            ServiceController serviceController = new ServiceController(DEFAULT_SERVICE_NAME);
            if (serviceController.Status != ServiceControllerStatus.Running)
            {
                serviceController.Start();
                serviceController.WaitForStatus(ServiceControllerStatus.Running,
                    TimeSpan.FromMilliseconds(DEFAULT_TIMEOUT));
            }

            OptionCollection collection = UnitTestHelper.CreateOptionCollection(
                new string[] { ApplicationServer.DEPLOYMENT_DIR, Server.TIMEOUT },
                new string[] { @"test_files/deploy", "-1" });
            server.Options = collection;
            server.ShutDown();
        }

        /// <summary>
        /// Tests the <c>ShutDown</c> when the TIMEOUT property is invalid, expect ServerException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestShutDown_InvalidTimeOutPropertyValue3()
        {
            InitializeServer();

            // start up the server
            ServiceController serviceController = new ServiceController(DEFAULT_SERVICE_NAME);
            if (serviceController.Status != ServiceControllerStatus.Running)
            {
                serviceController.Start();
                serviceController.WaitForStatus(ServiceControllerStatus.Running,
                    TimeSpan.FromMilliseconds(DEFAULT_TIMEOUT));
            }

            OptionCollection collection = UnitTestHelper.CreateOptionCollection(
                new string[] { ApplicationServer.DEPLOYMENT_DIR, Server.TIMEOUT },
                new string[] { @"test_files/deploy", "11111111111111111111111111111" });
            server.Options = collection;
            server.ShutDown();
        }

        /// <summary>
        /// Tests the <c>ShutDown</c> when the service name property is invalid, expect ServerException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestShutDown_InvalidServiceNamePropertyValue()
        {
            InitializeServer();

            // start up the server
            ServiceController serviceController = new ServiceController(DEFAULT_SERVICE_NAME);
            if (serviceController.Status != ServiceControllerStatus.Running)
            {
                serviceController.Start();
                serviceController.WaitForStatus(ServiceControllerStatus.Running,
                    TimeSpan.FromMilliseconds(DEFAULT_TIMEOUT));
            }

            OptionCollection collection = UnitTestHelper.CreateOptionCollection(
                new string[] { ApplicationServer.DEPLOYMENT_DIR, IISServer.SERVICE_NAME },
                new string[] { @"test_files/deploy", "Invalid" });
            server.Options = collection;
            server.ShutDown();
        }

        /// <summary>
        /// Tests the <c>ShutDown</c> when the service has already been stopped, expect ServerException.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestShutDown_AlreadyStopped()
        {
            InitializeServer();

            // start up the server
            ServiceController serviceController = new ServiceController(DEFAULT_SERVICE_NAME);
            if (serviceController.Status != ServiceControllerStatus.Running)
            {
                serviceController.Start();
                serviceController.WaitForStatus(ServiceControllerStatus.Running,
                    TimeSpan.FromMilliseconds(DEFAULT_TIMEOUT));
            }

            server.ShutDown();
            server.ShutDown();
        }

        /// <summary>
        /// Tests the accuracy of <c>ShutDown</c>.
        /// </summary>
        [Test]
        public void TestShutDown_Accuracy()
        {
            InitializeServer();

            // start up the server
            ServiceController serviceController = new ServiceController(DEFAULT_SERVICE_NAME);
            if (serviceController.Status != ServiceControllerStatus.Stopped)
            {
                serviceController.Stop();
                serviceController.WaitForStatus(ServiceControllerStatus.Stopped,
                    TimeSpan.FromMilliseconds(DEFAULT_TIMEOUT));
            }
            server.StartUp();

            server.ShutDown();

            // check that the server has stoped
            serviceController = new ServiceController(DEFAULT_SERVICE_NAME);
            Assert.AreEqual(ServiceControllerStatus.Stopped, serviceController.Status,
                "The service should be stopped.");

            // check the web-application has been deleted
            Assert.AreEqual(0, new DirectoryInfo(DES).GetFiles().Length,
                "The web-application should be deleted.");
        }

        /// <summary>
        /// Tests the accuracy of <c>InitData</c>.
        /// </summary>
        [Test]
        public void TestInitData_Accuracy()
        {
            server.InitData();
        }

        /// <summary>
        /// Tests the accuracy of <c>CleanUpData</c>.
        /// </summary>
        [Test]
        public void TestCleanUpData_Accuracy()
        {
            server.CleanUpData();
        }

        /// <summary>
        /// Initializes the server for testing. The required properties will be set.
        /// </summary>
        private void InitializeServer()
        {
            string deployDir = @"test_files/deploy";

            // set the option first
            OptionCollection collection = UnitTestHelper.CreateOptionCollection(
                new string[] { ApplicationServer.DEPLOYMENT_DIR }, new string[] { deployDir });
            server.Options = collection;

            // set the file set
            FileSet fileSet = new FileSet();
            Project project = new Project(@"../../test_files/FileSetElement.xml", Level.Info);
            fileSet.Project = project;
            fileSet.Initialize(UnitTestHelper.GetXmlNode(project.BuildFileLocalName, "fileset"));
            server.SourceFiles = fileSet;

            // set the project
            server.Project = project;
        }

        /// <summary>
        /// Checks the deploy result.
        /// </summary>
        private void CheckDeployResult()
        {
            IDictionary desFiles = new Hashtable();
            DirectoryInfo desDirectory = new DirectoryInfo(DES);
            foreach (FileInfo file in desDirectory.GetFiles())
            {
                desFiles.Add(file.Name, file.Name);
            }
            DirectoryInfo srcDirectory = new DirectoryInfo(SRC);
            foreach (FileInfo file in srcDirectory.GetFiles())
            {
                Assert.IsTrue(desFiles.Contains(file.Name), "The file of " + file.Name + " should be deployed.");
            }
        }
    }
}
